{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE DuplicateRecordFields, ScopedTypeVariables, TypeApplications #-}
{-# LANGUAGE OverloadedStrings #-}

{-# OPTIONS_GHC -Wno-orphans -Wno-incomplete-patterns #-}

module CFG.Parser
       ( parseFile
       ) where

import Data.Aeson
import qualified Data.ByteString.Lazy as BS
import qualified Data.IntSet as S

import CFG.Types

parseFile :: BS.ByteString -> Either String (Application ())
parseFile = eitherDecode'

instance FromJSON (Application ()) where
  parseJSON (Object v) = Application
    <$> v .: "functions"
    <*> (S.fromList <$> v .: "interrupts")

instance FromJSON (Function ()) where
  parseJSON (Object v) = Function
    <$> v .: "kind"
    <*> v .: "name"
    <*> v .: "subtask"
    <*> v .: "entry"
    <*> v .: "abbs"
    <*> pure ()

instance FromJSON FunctionKind where
  parseJSON (String "subtask") = pure KindSubtask
  parseJSON (String "function") = pure KindFunction
  parseJSON (String "isr") = pure KindISR

instance FromJSON Block where
  parseJSON (Object v) = Block
    <$> v .: "name"
    <*> v .: "successors"
    <*> v .: "call"
    <*> v .: "isKickoff"

instance FromJSON CallInfo where
  parseJSON (Object v) = do
    typ <- v .: "type"
    name <- v .: "name"

    case typ of
      String "syscall" -> return $ SystemCall name
      String "function" -> return $ FunctionCall name
