{-# LANGUAGE OverloadedStrings #-}

module CFG.Sanitize
  ( sanitizeNames
  , sanitize
  ) where

import           Data.Char
import           Data.List
import           Data.Maybe
import           Data.Monoid
import qualified Data.Text as T

import           CFG.Types

-- | Convert names from the CFG-Output to valid C identifiers
--
-- That entails removing e.g slashes and the like...
sanitizeNames :: Function () -> Function ()
sanitizeNames fun = fun { funName = sanitize (funName fun)
                        , funSubtask = sanitize (funSubtask fun)
                        , funEntryNode = sanitize (funEntryNode fun)
                        , funBlocks = map sanitizeBlock (funBlocks fun)
                        }

sanitizeBlock :: Block -> Block
sanitizeBlock blk = Block
  (sanitize (blkName blk))
  (map sanitize (blkSuccessors blk))
  (fmap sanitizeCallInfo (blkCallInfo blk))
  (blkIsKickoff blk)

sanitizeCallInfo :: CallInfo -> CallInfo
sanitizeCallInfo (FunctionCall t) = FunctionCall (sanitize t)
sanitizeCallInfo (SystemCall t) = SystemCall (sanitize t)

sanitize :: T.Text -> T.Text
sanitize = deunderscore . replaceIllegalChars

replaceIllegalChars :: T.Text -> T.Text
replaceIllegalChars = T.map replace
  where replace c | isAlphaNum c = c
                  | otherwise = '_'

deunderscore :: T.Text -> T.Text
deunderscore t
  | "_" `T.isPrefixOf` t   = "stub" <> t
  | otherwise              = t
