{-# LANGUAGE DeriveFunctor, DeriveTraversable #-}
module CFG.Types where

import Data.Text (Text)
import Data.Graph
import Control.Comonad
import Data.IntSet (IntSet)

data Application a = Application
  { appFunctions :: [Function a]
  , appInterrupts :: IntSet
  }
  deriving (Show, Eq, Functor, Foldable, Traversable)

data Function a = Function
  { funKind :: FunctionKind
  , funName :: Text
  , funSubtask :: Text
  , funEntryNode :: Text
  , funBlocks :: [Block]
  , funAnnotation :: a
  }
  deriving (Show, Eq, Functor, Foldable, Traversable)

type FunKleisli = Cokleisli Function

instance Comonad Function where
  extract = funAnnotation
  extend f x = let newAnno = f x in x { funAnnotation = newAnno }

data FunctionKind = KindFunction | KindSubtask | KindISR
  deriving (Show, Eq)

data Block = Block
  { blkName :: Text
  , blkSuccessors :: [Text]
  , blkCallInfo :: Maybe CallInfo
  , blkIsKickoff :: Bool
  }
  deriving (Show, Eq)

data CallInfo = FunctionCall Text | SystemCall Text
  deriving (Show, Eq)

data CFG = CFG
  { cfgGraph :: Graph
  , cfgGetBlock :: Vertex -> (Block, Text, [Text])
  , cfgGetVertex :: Text -> Maybe Vertex
  , cfgEntryVertex :: Vertex -- TODO Remove
  }

instance Show CFG where
  show = show . cfgGraph
