{-# LANGUAGE OverloadedStrings #-}


-- | Convert the output of the mockup to a state graph
--
-- Input is something like:
-- @
-- Line 192 in state 0x8d793408
-- Line 200 in state 0x8d793408
-- Decision 0: 0
-- Line 184 in state 0xb20f9937
-- Line 149 in state 0x1f2fb7b4
-- Line 141 in state 0x1f2fb7b4
-- @
module Search.States2Dot ( states2Dot ) where

import           Data.Maybe
import           Data.Monoid
import qualified Data.Text as T
import           Data.Text (Text)
import           Data.IntMap (IntMap)
import qualified Data.IntMap as M
import           Data.IntSet (IntSet)
import qualified Data.IntSet as S
import           Numeric

states2Dot :: Text -> Text
states2Dot = graph2Dot . states2Graph . input2States

-- FIXME Handle errors
input2States :: Text -> [[Int]]
input2States input =
  let individualRuns = T.splitOn "\n-\n" input
  in map (map (read . T.unpack) . mapMaybe parse . T.lines) individualRuns

  where
    parse x
      | "Line " `T.isPrefixOf` x = Just (last (T.words x))
      | otherwise                = Nothing

states2Graph :: [[Int]] -> IntMap IntSet
states2Graph = foldr (graph :: [Int] -> IntMap IntSet -> IntMap IntSet) M.empty
  where
    graph nodes imap = foldr insertNode imap (zip nodes (tail nodes))
    insertNode (from, to)
      | from == to  = id
      | otherwise   = M.insertWith S.union from (S.singleton to)

graph2Dot :: IntMap IntSet -> Text
graph2Dot graph =
     "digraph gcfg {\n"
  <> M.foldMapWithKey nodeStr graph
  <> "}"

  where
    nodeStr from tos = mconcat $ map (arrow from) $ S.toList tos
    arrow from to = "  " <> num2str from <> " -> " <> num2str to <> ";\n"
    num2str num = T.pack $ "x" ++ showHex num ""
