{-# LANGUAGE LambdaCase #-}

module Main where

import qualified Data.ByteString.Lazy as BS
import qualified Data.Text.IO as T
import           Options.Applicative
import           System.Exit
import           System.IO

import           CFG.Types
import qualified CFG.Parser as P
import           CFG.Graph
import           CFG.IR
import           CFG.C
import           CFG.Sanitize
import           Search.Search

data Command = Mockup MockupOpts | Search SearchOpts

optParser :: Parser Command
optParser = subparser
  (  command "mockup" (info (helper <*> (Mockup <$> mockupOptParser)) (progDesc "Generate mockup"))
  <> command "search" (info (helper <*> (Search <$> searchOptParser))
                       (progDesc "Extract CFG by running EXECUTABLE"))
  )

data SearchOpts = SearchOpts
  { mockupExecutable :: FilePath
  , identifyStates :: Bool
  }

searchOptParser :: Parser SearchOpts
searchOptParser = SearchOpts
  <$> argument str (metavar "EXECUTABLE")
  <*> switch ( long "identify-states"
            <> help "Identify equal states at different source locations"
             )

data MockupOpts = MockupOpts
  { triggerInterrupts :: Bool
  }

mockupOptParser :: Parser MockupOpts
mockupOptParser = MockupOpts
  <$> switch ( long "interrupts"
            <> help "Genenrate code to trigger interrupts"
             )

main :: IO ()
main = do
  cmd <- execParser $ info (helper <*> optParser)
            ( fullDesc <> progDesc "dOSEK verification tool" )
  case cmd of
    (Search opts) -> do
      if (identifyStates opts) then do
        hPutStrLn stderr "--identify-states is currently not implemented, sorry"
        exitFailure
      else
        driver (mockupExecutable opts)
    Mockup opts   -> do
      cntnt <- BS.getContents
      case P.parseFile cntnt of
        Left err -> putStrLn err
        Right res -> do
          let funs = compileToIRAll $ map (cfg . sanitizeNames) $ appFunctions res
          let ints = if triggerInterrupts opts then Just $ appInterrupts res else Nothing
          T.putStrLn $ compileToCAll ints $ map (fmap snd) funs
