{-# LANGUAGE OverloadedStrings, DuplicateRecordFields #-}
module CFG.GraphSpec (spec) where

import           Control.Monad
import           Data.Array
import           Data.Maybe
import           Data.Text (Text)
import qualified Data.Text as T
import           Test.Hspec

import           CFG.Graph
import           CFG.Types

spec :: Spec
spec = example1

example1Input :: Function ()
example1Input = Function
  { funKind = KindSubtask
  , funName = "Foo"
  , funSubtask = "Foo"
  , funEntryNode = "Two"
  , funBlocks =
    [ Block "Zero" [] Nothing False
    , Block "One" ["Two", "Zero"] Nothing False
    , Block "Two" ["One"] Nothing False
    ]
  , funAnnotation = ()
  }

example1 :: Spec
example1 = describe "graphify" $ do
  let cfg = graphify example1Input
      hasSuccs = hasSuccessors cfg

  it "works with an example" $ do
    bounds (cfgGraph cfg) `shouldBe` (0, 2)
    "Zero" `hasSuccs` []
    "One" `hasSuccs` ["Two", "Zero"]
    "Two" `hasSuccs` ["One"]
    cfgEntryVertex cfg `shouldBe` fromJust (cfgGetVertex cfg "Two")

hasSuccessors :: CFG -> Text -> [Text] -> Expectation
hasSuccessors cfg node succs =
  let nodeVertex = cfgGetVertex cfg node
      succVertices = map (cfgGetVertex cfg) succs
  in do
    forM_ (zip (nodeVertex : succVertices) (node:succs)) $ \(vert, name) ->
      vert `shouldSatisfy` isJust
    (cfgGraph cfg ! fromJust nodeVertex) `shouldBe` map fromJust succVertices
