{-# LANGUAGE OverloadedStrings, DuplicateRecordFields #-}
module CFG.IRSpec (spec) where

import           Data.Function
import           Data.Graph
import           Data.List
import           Data.Maybe
import           Data.Text (Text)
import           Test.Hspec
import           Control.Comonad

import           CFG.Graph
import           CFG.IR
import           CFG.Types

spec :: Spec
spec = do
  example1

example1Input :: Function ()
example1Input = Function
  { funKind = KindSubtask
  , funName = "Foo"
  , funSubtask = "Foo"
  , funEntryNode = "Two"
  , funBlocks =
    [ Block "Zero" [] Nothing False
    , Block "One" ["Two", "Zero"] (Just $ FunctionCall "foo") False
    , Block "Two" ["One"] (Just $ SystemCall "bar") False
    ]
  , funAnnotation = ()
  }

example1Output :: CFG -> [Instr]
example1Output cfg = map snd $
  -- We don't know in which order the blocks are translated to vertices by
  -- graphify. So we bring this in the right order by sorting on the
  -- vertex-number.
  sortBy (compare `on` fst)
    [ (-1, Goto "Two")
    , (vert "Zero", Label "Zero")
    , (vert "Zero", Return)
    , (vert "One", Label "One")
    , (vert "One", Call "foo" TypeFunction False)
    , (vert "One", IfThanElse "Two" "Zero")
    , (vert "Two", Label "Two")
    , (vert "Two", Call "bar" TypeSyscall False)
    , (vert "Two", Goto "One")
    ]

  where vert :: Text -> Vertex
        vert = fromJust . cfgGetVertex cfg

example1 :: Spec
example1 = describe "flattenCFG" $ do
  let fun = extend graphify example1Input
  it "works with an example" $
    flattenCFG fun `shouldBe` example1Output (extract fun)
